#include    "LCD_Display_1.h"
#include	"PI-250_PCB.h"
#include    <stdio.h>   //for sprintf
#include    <stdlib.h>  //for abs
#include    <string.h>  //for strlen
#include    <math.h>

char            DispMsg[17];

void    LCD_Clear_Display(void)
{
	LCD_Write_Command( LCD_CLEAR );		//Display clear
	__delay_ms(2);   
}
/******************** Write integer value to display ******************/

void	LCD_WriteMsgInt( char row, char col, int val )
	{
	sprintf( DispMsg, "%04d", val );
	LCD_WriteMsg( row, col, DispMsg );
		}

/******************** Write message to LCD display ******************/

void	LCD_WriteMsg( char row, char col, char *msg )
	{
	int		msgptr, msglen, cmd;
	cmd = 0b10000000 | col | (row<<6);
	msglen = strlen(msg);
	LCD_Write_Command( cmd );
	for( msgptr=0; msgptr<msglen; msgptr++)
		LCD_Write_Data(msg[msgptr]);
	}

/******************** Write float value to display ******************
1000 - 9999		No DP
100.0 - 999.9	DP2
10.00 - 99.99	DP3
0.000 - 9.999	DP4
********************************************************************/

void	LCD_WriteMsgFloat( char row, char col, char prec, float fval )
	{
	int		ival, mult;

	if(fval < 0) {
		sprintf(DispMsg, "-" );
		LCD_WriteMsg( row, col, DispMsg );
		if(prec>0)
			prec--;
		}
	fval = fabs(fval);
	mult = 1;
	for( ival=1; ival<=prec; ival++ )
		mult *= 10;
	if( fval>=0 )
		ival = (int)floor(fval);
	else
		ival = (int)ceil(fval);
	sprintf(DispMsg, "%*d", 4-prec, ival);
	LCD_WriteMsg( row, col, DispMsg );
	if(prec>0) {
		LCD_Write_Data( '.' );
		fval = mult*(fval-ival);    //+0.5/mult - changed 11/13/12 PES
		ival = abs((int)floor(fval));
		col += (strlen(DispMsg)+1);
		sprintf(DispMsg, "%0*d", prec, ival);
		LCD_WriteMsg( row, col, DispMsg );
		}
    else
		LCD_Write_Data( ' ' );
	}

void	LCD_Write_Command(char CMD)
	{
	LCD_RS = 0;
	__delay_us(5);
	LCD_DB = CMD ;					//Load data bus
	__delay_us(25);                 // 5 us too fast
	LCD_E = 1;
	__delay_us(5);
	LCD_E = 0;
	__delay_us(5);
	}

void	LCD_Write_Data(char DATA)
	{
	LCD_RS = 1;			//set for data
	__delay_us(5);
	LCD_DB = DATA;					//Load data bus
	__delay_us(25);                 // 5 uS too fast
	LCD_E = 1;
	__delay_us(5);
	LCD_E = 0;
	__delay_us(5);
	}

/*
 * Initialization for 8 bit control
 * 
 * Power ON
 * Delay > 40 mSec after 2.7 volts
 * Function Set 8 bits 0b00110000
 * Delay > 4.1 mSec
 * Function Set 8 bits 0b00110000
 * Delay > 100 uSec
 * Function Set 8 bits 0b00110000
 * Delay > 40 uSec
 * Function Set Display Lines and Font 0b0011NF00
 * Delay > 40 uSec
 * Function Set Display Off 0b00001000
 * Delay > 40 uSec
 * Function Set Display Clear 0b00000001
 * Delay > 2 mSec
 * Function Set Display Entry Mode 0b 0 0 0 0  0 1 I/D S I=1 (Increment) S=1 (Cursor Shift)
 * Delay > 40 uSec
 * Initialization end
 * Function Set Display Home 0b00000010
 * Delay > 2 mSec
 * Function Set Display ON 0b00001110  0000 1 [D]isplay [C]ursor [B]link
 * Delay > 40 uSec
 */
void	LCD_Init_Display(void)
	{
    Delay_mSec(200);
	LCD_RW = 0;		//Set for write
	Delay_mSec(5);                     // was 20
	LCD_Write_Command( 0b00110000);		//Function set 8 bit
	Delay_mSec(6);						//Need at least 4.1 mSec
	LCD_Write_Command( 0b00110000 );	//Function set 8 bit
	__delay_us(150);					//Need at least 100 uSec
	LCD_Write_Command( 0b00110000 );	//Function set 8 bit
	__delay_us(50);
			//0 0 1 DL N F x x	;DL: 1=8Bit 0=4Bit, N: 1=2Line 0=1Line, F: 1=10Dot 0=8Dot
	LCD_Write_Command( 0b00111000 );	//8 bit Function set 2 lines, 5x8 dots
	__delay_us(50);
	LCD_Write_Command( 0b00001000 );	//Display off
	__delay_us(50);
	LCD_Write_Command( 0b00000001 );		//Display clear
	Delay_mSec(2);
	LCD_Write_Command( 0b00000110 );	//Entry mode set, <1> 1=incr, <0> 0=no shift
	__delay_us(50);
	LCD_Write_Command( 0b00010100 );	//Shift cursor right
	__delay_us(50);
	LCD_Write_Command( 0b00000010	);		//Cursor home
	Delay_mSec(2);
		// Cursor is underline, Blink flashes character ;Display ON/OFF, Cursor OFF, Blink OFF
	LCD_Write_Command( 0b00001100 );
	Delay_mSec(20);						//long delay
	}

